use axum::{
    extract::{Path, State},
    response::Json,
    routing::{get, post, delete, put},
    Router,
};
use sea_orm::DatabaseConnection;
use serde_json::{json, Value};

use crate::error::ApiError;
use super::service::BaseService;

pub struct BaseController<S: BaseService> {
    service: S,
}

impl<S: BaseService + Send + Sync + 'static> BaseController<S> {
    pub fn new(service: S) -> Self {
        Self { service }
    }

    pub fn routes(&self) -> Router<DatabaseConnection> {
        Router::new()
            .route("/", get(Self::find_all))
            .route("/", post(Self::create))
            .route("/:id", get(Self::find_by_id))
            .route("/:id", put(Self::update))
            .route("/:id", delete(Self::delete))
    }

    async fn find_all(
        State(db): State<DatabaseConnection>,
        service: S,
    ) -> Result<Json<Value>, ApiError> {
        let items = service.find_all(&db).await?;
        Ok(Json(json!({ "data": items })))
    }

    async fn find_by_id(
        State(db): State<DatabaseConnection>,
        Path(id): Path<i32>,
        service: S,
    ) -> Result<Json<Value>, ApiError> {
        let item = service.find_by_id(&db, id).await?;
        Ok(Json(json!({ "data": item })))
    }

    // Additional CRUD methods implementation...
}
