use async_trait::async_trait;
use sea_orm::{DatabaseConnection, EntityTrait};

use crate::error::ApiError;

#[async_trait]
pub trait BaseService {
    type Entity: EntityTrait;

    async fn find_all(&self, db: &DatabaseConnection) -> Result<Vec<Self::Entity::Model>, ApiError>;
    async fn find_by_id(&self, db: &DatabaseConnection, id: i32) -> Result<Vec<Self::Entity::Model>, ApiError>;
    async fn create(&self, db: &DatabaseConnection, data: <Self::Entity as EntityTrait>::Model) -> Result<Self::Entity::Model, ApiError>;
    async fn update(&self, db: &DatabaseConnection, id: i32, data: <Self::Entity as EntityTrait>::Model) -> Result<Self::Entity::Model, ApiError>;
    async fn delete(&self, db: &DatabaseConnection, id: i32) -> Result<(), ApiError>;
}
